# frozen_string_literal: true

module ElasticsearchHermes
  class Index
    attr_reader :name

    def initialize(name:, fields: [])
      @name = name
      @fields = fields
    end

    def create
      ElasticsearchHermes.client.indices.create index: @name, body: index_options
    end

    def delete
      ElasticsearchHermes.client.indices.delete index: @name
    end

    def exist?
      ElasticsearchHermes.client.indices.exists? index: @name
    end

    private

    def build_field_mappings
      properties = {}
      analyzer = {}
      tokenizer = {}
      @fields.each do |field|
        # add checks for full_match
        if field[:search_type] == :partial_match
          @field_token = ngram_builder(field[:minimum_match], field[:field_name])
          @field_analyzer = analyzer_builder(field[:field_name], 'custom', "#{field[:field_name]}_tokens", 'lowercase')
        end

        prop = {
          "#{field[:field_name]}": {
            type: field[:field_type],
            analyzer: "#{field[:field_name]}_analyzer"
          }
        }

        analyzer = analyzer.merge(@field_analyzer || {})
        tokenizer = tokenizer.merge(@field_token || {})
        properties = properties.merge(prop || {})
      end

      { properties: properties, analyzer: analyzer, tokenizer: tokenizer }
    end

    def index_options
      field_mappings = build_field_mappings
      {
        settings: {
          analysis: {
            analyzer: field_mappings[:analyzer],
            tokenizer: field_mappings[:tokenizer]
          }
        },
        mappings: {
          properties: field_mappings[:properties]
        }
      }
    end

    def field_mappings_builder(field_name, field_type, type_analyzer)
      {
        "#{field_name}": {
          type: field_type,
          analyzer: type_analyzer
        }
      }
    end

    def analyzer_builder(analyzer_name, analyzer_type, tokenizer, filter)
      {
        "#{analyzer_name}_analyzer": {
          type: analyzer_type,
          tokenizer: tokenizer,
          filter: filter
        }
      }
    end

    def ngram_builder(number, field_name)
      {
        "#{field_name}_tokens": {
          type: 'ngram',
          min_gram: number,
          max_gram: number,
          token_chars: [
            'letter'
          ]
        }
      }
    end
  end
end
